unit FormRgn;
{ *************************************************
  *            FormRgn library 04.05.99           *
  *           written by Grigoriev Anton          *
  *                                               *
  *                    Main unit                  *
  *   This unit must be installed for the whole   *
  *                 library to work               *
  ************************************************* }

 interface

  uses Windows,Messages,SysUtils,Classes,Graphics,Controls,Forms,Dialogs,DsgnIntf,Menus,RgnEdit,ExtCtrls,Math;

   const WM_ExtRgnChanged=WM_User+1216;
         WM_ExtRgnNewName=WM_User+1217;

    type EOwnerMustBeForm=class(Exception);
         ECannotTranslate=class(Exception);
         EDuplicateComponent=class(Exception);
         EOwnerNotAssigned=class(Exception);

         TSizeChangedEvent=procedure(Sender:TObject;var Region:string) of object;

         TExternalRegion=class;

         TRegionComp=class(TComponent)
                      private
                       FRegion:string;
                       Rgn:HRgn;
                       OldWndProc:TFarProc;
                       NewWndProc:Pointer;
                       ExtRgn:TExternalRegion;
                       procedure SetRegion(const Value:string);
                       procedure HookOwner;
                       procedure UnhookOwner;
                      protected
                       property Region:string read FRegion write SetRegion;
                       function CreateRegion:Boolean;dynamic;
                       destructor Destroy;override;
                       procedure Loaded;override;
                       function GetXOrigin:Integer;
                       function GetYOrigin:Integer;
                       function DesignWidth:Integer;
                       function DesignHeight:Integer;
                       procedure CallDefault(var Message:TMessage);
                       procedure RegionCreated;dynamic;abstract;
                       procedure DoDestroy;dynamic;abstract;
                       procedure HookWndProc(var Message:TMessage);virtual;
                       procedure Notification(AComponent:TComponent;Operation:TOperation);override;
                       procedure Activate;dynamic;
                       procedure Deactivate;dynamic;
                      public
                       constructor Create(AOwner:TComponent);override;
                     end;

         TFormRegion=class(TRegionComp)
                      private
                       FDrawOnDesigning:Boolean;
                       FClientDrag:Boolean;
                       FOnSizeChanged:TSizeChangedEvent;
                       procedure SetDrawOnDesigning(Value:Boolean);
                      protected
                       procedure RegionCreated;override;
                       procedure DoDestroy;override;
                       procedure HookWndProc(var Message:TMessage);override;
                      public
                       constructor Create(AOwner:TComponent);override;
                      published
                       property ClientDrag:Boolean read FClientDrag write FClientDrag default False;
                       property DrawOnDesigning:Boolean read FDrawOnDesigning write SetDrawOnDesigning default True;
                       property OnSizeChanged:TSizeChangedEvent read FOnSizeChanged write FOnSizeChanged default nil;
                       property Region;
                     end;

         TCaptionRegion=class;

         TCaptionLabel=class(TCollectionItem)
                        private
                         FColors:array[0..1] of TColor;
                         FText:string;
                         FFont:TFont;
                         FCoords:array[0..2] of Integer;
                         FVisible:Boolean;
                         function GetDisplayName:string;override;
                         procedure SetColors(Index:Integer;Value:TColor);
                         procedure SetText(Value:string);
                         procedure SetFont(Value:TFont);
                         procedure SetCoords(Index:Integer;Value:Integer);
                         procedure SetVisible(Value:Boolean);
                         procedure FontChanged(Sender:TObject);
                        protected
                         destructor Destroy;override;
                        public
                         constructor Create(Collection:TCollection);override;
                         procedure Assign(Source:TPersistent);override;
                         procedure Draw(DC:HDC;CIndex:Integer);
                        published
                         property ActiveColor:TColor index 1 read FColors[1] write SetColors default clCaptionText;
                         property Angle:Integer index 2 read FCoords[2] write SetCoords default 0;
                         property Font:TFont read FFont write SetFont;
                         property InactiveColor:TColor index 0 read FColors[0] write SetColors default clInactiveCaptionText;
                         property Left:Integer index 0 read FCoords[0] write SetCoords default 0;
                         property Text:string read FText write SetText;
                         property Top:Integer index 1 read FCoords[1] write SetCoords default 0;
                         property Visible:Boolean read FVisible write SetVisible default True;
                       end;

         TCaptionLabels=class(TCollection)
                         private
                          CaptionRegion:TCaptionRegion;
                          function GetItem(Index:Integer):TCaptionLabel;
                          procedure SetItem(Index:Integer;Value:TCaptionLabel);
                         protected
                          function GetOwner:TPersistent;override;
                          procedure Update(Item:TCollectionItem);override;
                         public
                          constructor Create(ACaptionRegion:TCaptionRegion);
                          function Add:TCaptionLabel;
                          procedure Draw(DC:HDC;CIndex:Integer);
                          property Items[Index:Integer]:TCaptionLabel read GetItem write SetItem;default;
                        end;

         TSysMenuPicture=class(TPersistent)
                          private
                           FGlyph:TBitmap;
                           Owner:TCaptionRegion;
                           FCoords:array[0..1] of Integer;
                           FFlags:array[0..1] of Boolean;
                           FSize:array[0..1] of Integer;
                           procedure SetGlyph(Value:TBitmap);
                           procedure SetCoords(Index:Integer;Value:Integer);
                           procedure SetFlags(Index:Integer;Value:Boolean);
                           procedure SetSize(Index:Integer;Value:Integer);
                           procedure Redraw;
                          protected
                           destructor Destroy;override;
                          public
                           constructor Create(AOwner:TCaptionRegion);
                           procedure Assign(Source:TPersistent);override;
                           procedure Draw(DC:HDC);
                           function RealWidth:Integer;
                           function RealHeight:Integer;
                          published
                           property Glyph:TBitmap read FGlyph write SetGlyph;
                           property Height:Integer index 0 read FSize[0] write SetSize default -1;
                           property Left:Integer index 0 read FCoords[0] write SetCoords default 0;
                           property Top:Integer index 1 read FCoords[1] write SetCoords default 0;
                           property Transparent:Boolean index 0 read FFlags[0] write SetFlags default True;
                           property Visible:Boolean index 1 read FFlags[1] write SetFlags default False;
                           property Width:Integer index 1 read FSize[1] write SetSize default -1;
                         end;

         TCaptionRegion=class(TRegionComp)
                         private
                          FColors:array [0..1] of TColor;
                          FOnSizeChanged:TSizeChangedEvent;
                          FPopupMenu:TPopupMenu;
                          FSysMenuPicture:TSysMenuPicture;
                          ClickTime:Integer;
                          FLabels:TCaptionLabels;
                          procedure SetColor(Index:Integer;Value:TColor);
                          procedure SetSysMenuPicture(Value:TSysMenuPicture);
                          procedure SetLabels(Value:TCaptionLabels);
                         protected
                          procedure HookWndProc(var Message:TMessage);override;
                          procedure RegionCreated;override;
                          procedure DoDestroy;override;
                          procedure Loaded;override;
                         public
                          constructor Create(AOwner:TComponent);override;
                         published
                          property ActiveColor:TColor index 1 read FColors[1] write SetColor default clActiveCaption;
                          property InactiveColor:TColor index 0 read FColors[0] write SetColor default clInactiveCaption;
                          property Labels:TCaptionLabels read FLabels write SetLabels;
                          property OnSizeChanged:TSizeChangedEvent read FOnSizeChanged write FOnSizeChanged default nil;
                          property PopupMenu:TPopupMenu read FPopupMenu write FPopupMenu default nil;
                          property SysMenuPicture:TSysMenuPicture read FSysMenuPicture write SetSysMenuPicture;
                          property Region;
                        end;

         TExternalRegion=class(TComponent)
                          private
                           Next:TExternalRegion;
                           First:Boolean;
                           FOwnRegion:Boolean;
                           Rgn:HRgn;
                           FOldName:TComponentName;
                          protected
                           destructor Destroy;override;
                           procedure Notification(AComponent:TComponent;Operation:TOperation);override;
                           function MakeRgn:HRgn;virtual;abstract;
                           procedure RgnChanged;dynamic;
                           procedure SetName(const Value:TComponentName);override;
                           function RgnSize:TSize;dynamic;
                          public
                           constructor Create(AOwner:TComponent);override;
                           function GetRgn:HRgn;
                          published
                           property OwnRegion:Boolean read FOwnRegion write FOwnRegion default True;
                         end;

         TTextExtRgn=class(TExternalRegion)
                      private
                       FText:string;
                       FAngle:Integer;
                       FWeight:Integer;
                       FFont:TFont;
                       FTransparent:Boolean;
                       procedure SetText(const Value:string);
                       procedure SetAngle(Value:Integer);
                       procedure SetWeight(Value:Integer);
                       procedure SetFont(Value:TFont);
                       procedure SetTransparent(Value:Boolean);
                       procedure FontChanged(Sender:TObject);
                      protected
                       destructor Destroy;override;
                       function MakeRgn:HRgn;override;
                      public
                       constructor Create(AOwner:TComponent);override;
                      published
                       property Angle:Integer read FAngle write SetAngle default 0;
                       property Font:TFont read FFont write SetFont;
                       property Text:string read FText write SetText;
                       property Transparent:Boolean read FTransparent write SetTransparent default True;
                       property Weight:Integer read FWeight write SetWeight default -1;
                     end;

         TRegionAlignment=(raResult,raSource);

         TPictExtRgn=class(TExternalRegion)
                      private
                       FGlyph:TBitmap;
                       FTransColor:TColor;
                       FInverted:Boolean;
                       FAlign:array[0..1] of TRegionAlignment;
                       procedure SetGlyph(Value:TBitmap);
                       procedure SetTransColor(Value:TColor);
                       procedure SetInverted(Value:Boolean);
                       procedure SetAlign(Index:Integer;Value:TRegionAlignment);
                      protected
                       destructor Destroy;override;
                       function MakeRgn:HRgn;override;
                       function BitmapToRgn(Bitmap:TBitmap):HRgn;
                       function RgnSize:TSize;override;
                      public
                       constructor Create(AOwner:TComponent);override;
                      published
                       property Glyph:TBitmap read FGlyph write SetGlyph;
                       property HorAlign:TRegionAlignment index 0 read FAlign[0] write SetAlign default raResult;
                       property Inverted:Boolean read FInverted write SetInverted default False;
                       property TransColor:TColor read FTransColor write SetTransColor default clOlive;
                       property VertAlign:TRegionAlignment index 1 read FAlign[1] write SetAlign default raResult;
                     end;

         THandlingControl=class(TCustomControl)
                           private
                            OldWndProc:TFarProc;
                            NewWndProc:Pointer;
                            procedure HookOwner;
                            procedure UnhookOwner;
                           protected
                            procedure CallDefault(var Message:TMessage);
                            procedure HookWndProc(var Message:TMessage);virtual;
                            destructor Destroy;override;
                            procedure Activate;dynamic;
                            procedure Deactivate;dynamic;
                           public
                            constructor Create(AOwner:TComponent);override;
                          end;

         TMenuHotSpot=class(THandlingControl)
                       private
                        FGlyph:TBitmap;
                        FTransparent:Boolean;
                        FColors:array[0..1] of TColor;
                        ClickTime:Integer;
                        procedure SetGlyph(Value:TBitmap);
                        procedure SetTransparent(Value:Boolean);
                        procedure SetColor(Index:Integer;Value:TColor);
                       protected
                        destructor Destroy;override;
                        procedure MouseDown(Button:TMouseButton;Shift:TShiftState;X,Y:Integer);override;
                        procedure DblClick;override;
                        procedure Paint;override;
                        procedure HookWndProc(var Message:TMessage);override;
                       public
                        constructor Create(AOwner:TComponent);override;
                       published
                        property ActiveColor:TColor index 1 read FColors[1] write SetColor default clActiveCaption;
                        property Glyph:TBitmap read FGlyph write SetGlyph;
                        property InactiveColor:TColor index 0 read FColors[0] write SetColor default clInactiveCaption;
                        property PopupMenu;
                        property Transparent:Boolean read FTransparent write SetTransparent;
                        property Visible;
                        property ShowHint;
                        property ParentShowHint;
                      end;

         TSysButtonKind=(sbClose,sbMinimize,sbMaximize,sbHelp,sbGlyph,sbOwnerDraw,sbOwnerDrawFace);

         TSystemButton=class(THandlingControl)
                        private
                         FKind:TSysButtonKind;
                         FAutoSize:Boolean;
                         FOnDrawButton:TNotifyEvent;
                         FPressed:Boolean;
                         FPressLoop:Boolean;
                         FGlyph:TBitmap;
                         procedure SetKind(Value:TSysButtonKind);
                         procedure SetAutoSize(Value:Boolean);
                         procedure SetGlyph(Value:TBitmap);
                         procedure CMEnabledChanged(var Message:TMessage);message CM_EnabledChanged;
                        protected
                         destructor Destroy;override;
                         procedure Paint;override;
                         procedure MouseDown(Button:TMouseButton;Shift:TShiftState;X,Y:Integer);override;
                         procedure MouseMove(Shift:TShiftState;X,Y:Integer);override;
                         procedure MouseUp(Button:TMouseButton;Shift:TShiftState;X,Y:Integer);override;
                         procedure Loaded;override;
                         procedure HookWndProc(var Message:TMessage);override;
                        public
                         property Pressed:Boolean read FPressed;
                         property Canvas;
                         constructor Create(AOwner:TComponent);override;
                         procedure SetBounds(ALeft,ATop,AWidth,AHeight:Integer);override;
                        published
                         property AutoSize:Boolean read FAutoSize write SetAutoSize default True;
                         property Glyph:TBitmap read FGlyph write SetGlyph;
                         property Kind:TSysButtonKind read FKind write SetKind default sbClose;
                         property Enabled;
                         property Visible;
                         property ShowHint;
                         property ParentShowHint;
                         property OnClick;
                         property OnDrawButton:TNotifyEvent read FOnDrawButton write FOnDrawButton default nil;
                       end;

         TCaptionText=class(TGraphicControl)
                       private
                        FAngle:Integer;
                        FColors:array[0..1] of TColor;
                        FAutoSize:Boolean;
                        OldWndProc:TFarProc;
                        NewWndProc:Pointer;
                        procedure HookOwner;
                        procedure UnhookOwner;
                        procedure SetAngle(Value:Integer);
                        procedure SetAutoSize(Value:Boolean);
                        procedure SetColor(Index:Integer;Value:TColor);
                       protected
                        destructor Destroy;override;
                        procedure CallDefault(var Message:TMessage);
                        procedure HookWndProc(var Message:TMessage);virtual;
                        function CalcSize:TSize;
                        procedure SetSize;
                        procedure CMTextChanged(var Message:TMessage);message CM_TextChanged;
                        procedure CMFontChanged(var Message:TMessage);message CM_FontChanged;
                        procedure Paint;override;
                        procedure Activate;dynamic;
                        procedure Deactivate;dynamic;
                       public
                        constructor Create(AOwner:TComponent);override;
                       published
                        property ActiveColor:TColor index 1 read FColors[1] write SetColor default clCaptionText;
                        property Angle:Integer read FAngle write SetAngle default 0;
                        property AutoSize:Boolean read FAutoSize write SetAutoSize default True;
                        property InactiveColor:TColor index 0 read FColors[0] write SetColor default clInactiveCaptionText;
                        property Caption;
                        property Visible;
                        property Font;
                      end;

    TFRManager=class(TComponent)
                private
                 CallCount:Integer;
                public
                 constructor Create(AOwner:TComponent);override;
                 procedure Activate;dynamic;
                 procedure Deactivate;dynamic;
                 procedure StayOnTop(Stay:Boolean);
               end;

    procedure Register;

 implementation

  type PPointsArray=^TPointsArray;
       TPointsArray=array[1..1000000] of TPoint;

   var DFrame,SFrame,SzFrame:TPoint;
       NCapt,SmCapt:Integer;

    {TRegionComp}

    constructor TRegionComp.Create;
     begin
      ExtRgn:=nil;
      if not (AOwner is TForm) then
       raise EOwnerMustBeForm.Create('The owner of '+ClassName+' component must be the TForm descendant');
      inherited Create(AOwner);
      FRegion:='';
      Rgn:=0;
      HookOwner
     end;

    destructor TRegionComp.Destroy;
     begin
      UnhookOwner;
      DoDestroy;
      inherited Destroy
     end;

    procedure TRegionComp.Loaded;
     begin
      inherited Loaded;
      Region:=Region
     end;

    procedure TRegionComp.HookOwner;
     begin
      if not Assigned(Owner) then
       Exit;
      OldWndProc:=TFarProc(GetWindowLong((Owner as TForm).Handle,GWL_WndProc));
      NewWndProc:=MakeObjectInstance(HookWndProc);
      SetWindowLong((Owner as TForm).Handle,GWL_WndProc,LongInt(NewWndProc))
     end;

    procedure TRegionComp.UnhookOwner;
     begin
      if Assigned(Owner) and Assigned(OldWndProc) then
       SetWindowLong((Owner as TForm).Handle,GWL_WndProc,LongInt(OldWndProc));
      if Assigned(NewWndProc) then
       FreeObjectInstance(NewWndProc);
      NewWndProc:=nil;
      OldWndProc:=nil
     end;

    procedure TRegionComp.HookWndProc;
     var ER:TExternalRegion;
         OldName,NewRgn:string;
         P:Integer;
      begin
       if Message.Msg=WM_ExtRgnChanged then
        if Pos(UpperCase(TExternalRegion(Message.LParam).Name),UpperCase(FRegion))<>0 then
         Region:=Region;
       if Message.Msg=WM_ExtRgnNewName then
        begin
         ER:=TExternalRegion(Message.LParam);
         OldName:=UpperCase(ER.FOldName);
         if OldName<>'' then
          begin
           NewRgn:=FRegion;
           P:=Pos(OldName,UpperCase(NewRgn));
           while P<>0 do
            begin
             Delete(NewRgn,P,Length(OldName));
             Insert(ER.Name,NewRgn,P);
             P:=Pos(OldName,UpperCase(NewRgn))
            end;
           Region:=NewRgn
          end
        end;
       CallDefault(Message)
      end;

    procedure TRegionComp.SetRegion;
     begin
      FRegion:=Value;
      if not (csLoading in ComponentState) then
       if CreateRegion then
        RegionCreated
     end;

    function TRegionComp.CreateRegion;
     var S:string;
         Comb:array[1..15] of Integer;
         P,N,C:Integer;
         R2:array[1..16] of HRgn;
         First:array[1..15] of Boolean;

      procedure ProcessString(S:string);
       var I,P,CN:Integer;
           RR:array[1..6] of Integer;
           Ch,ChV,ChH:Char;
           Pts:PPointsArray;
           PE:TExternalRegion;
           R:TRect;
           X,Y:Integer;
           Found:Boolean;
           Sz:TSize;

        function Trans(var P:Integer;const S:string):Integer;
         forward;

        function Factor(var P:Integer;const S:string):Integer;
         forward;

        function Expr(var P:Integer;const S:string):Integer;
         begin
          Result:=Trans(P,S);
          while S[P] in ['+','-'] do
           case S[P] of
            '+':begin
                 Inc(P);
                 Inc(Result,Trans(P,S))
                end;
            '-':begin
                 Inc(P);
                 Dec(Result,Trans(P,S))
                end
           end
         end;

        function Trans;
         begin
          Result:=Factor(P,S);
          while S[P] in ['*','/'] do
           case S[P] of
            '*':begin
                 Inc(P);
                 Result:=Result*Factor(P,S);
                end;
            '/':begin
                 Inc(P);
                 Result:=Result div Factor(P,S)
                end
           end
         end;

        function Factor;
         var J,D:Integer;
          begin
           case UpCase(S[P]) of
            '(':begin
                 Inc(P);
                 Result:=Expr(P,S);
                 if S[P]=')' then
                  Inc(P)
                 else
                  raise ECannotTranslate(''')'' expected in position '+IntToStr(P))
                end;
            '-':begin
                 Inc(P);
                 Result:=-Factor(P,S)
                end;
            '0'..'9':begin
                      Val(Copy(S,P,255),Result,D);
                      J:=D;
                      Val(Copy(S,P,J-1),Result,D);
                      P:=P+J-1
                     end;
            'B':begin
                 Inc(P);
                 with Owner as TForm do
                  Result:=ClientHeight+ClientOrigin.Y-Top-GetYOrigin
                end;
            'C':begin
                 Inc(P);
                 with Owner as TForm do
                  Result:=ClientOrigin.Y-Top-GetYOrigin
                end;
            'H':begin
                 Inc(P);
                 Result:=(Owner as TForm).Height-DesignHeight
                end;
            'R':begin
                 Inc(P);
                 with Owner as TForm do
                  Result:=ClientWidth+ClientOrigin.X-Left-GetXOrigin
                end;
            'W':begin
                 Inc(P);
                 Result:=(Owner as TForm).Width-DesignWidth
                end;
            'X':begin
                 Inc(P);
                 with Owner as TForm do
                  Result:=ClientOrigin.X-Left-GetXOrigin
                end;
            'Y':begin
                 Inc(P);
                 with Owner as TForm do
                  if BorderStyle in [bsToolWindow,bsSizeToolWin] then
                   Result:=ClientOrigin.Y-Top-SmCapt-GetYOrigin
                  else
                   Result:=ClientOrigin.Y-Top-NCapt-GetYOrigin
                end
           else
            raise ECannotTranslate.Create('Syntax error in position '+IntToStr(P))
           end
          end;

        function GetValue(const S:string):Integer;
         var P:Integer;
          begin
           P:=1;
           Result:=Expr(P,S+' ')
          end;

        function MakeRect(S:string):TRect;
         var I,CN:Integer;
          begin
           CN:=0;
           for I:=1 to Length(S) do
            if S[I]=',' then
             Inc(CN);
           if CN<>3 then
            raise ECannotTranslate.Create('Invalid number of parameters');
           I:=Pos(',',S);
           Result.Left:=GetValue(Copy(S,1,I-1));
           Delete(S,1,I);
           I:=Pos(',',S);
           Result.Top:=GetValue(Copy(S,1,I-1));
           Delete(S,1,I);
           I:=Pos(',',S);
           Result.Right:=GetValue(Copy(S,1,I-1));
           Delete(S,1,I);
           Result.Bottom:=GetValue(S)
          end;

         begin
          R2[C+1]:=0;
          Ch:=S[1];
          Delete(S,1,1);
          case Ch of
           'A':Comb[C]:=Rgn_And;
           'D':Comb[C]:=Rgn_Diff;
           'E':R2[C+1]:=CreateEllipticRgnIndirect(MakeRect(S));
           'L','W':begin
                    CN:=0;
                    for I:=1 to Length(S) do
                     if S[I]=',' then
                      Inc(CN);
                    if (CN<5) or not Odd(CN) then
                     raise ECannotTranslate.Create('Invalid number of parameters');
                    CN:=(CN+1) div 2;
                    GetMem(Pts,SizeOf(TPoint)*CN);
                    try
                     for I:=1 to CN-1 do
                      begin
                       P:=Pos(',',S);
                       Pts^[I].X:=GetValue(Copy(S,1,P-1));
                       Delete(S,1,P);
                       P:=Pos(',',S);
                       Pts^[I].Y:=GetValue(Copy(S,1,P-1));
                       Delete(S,1,P)
                      end;
                     P:=Pos(',',S);
                     Pts^[CN].X:=GetValue(Copy(S,1,P-1));
                     Delete(S,1,P);
                     Pts^[CN].Y:=GetValue(S);
                     if Ch='L' then
                      R2[C+1]:=CreatePolygonRgn(Pts^,CN,Alternate)
                     else
                      R2[C+1]:=CreatePolygonRgn(Pts^,CN,Winding)
                    finally
                     FreeMem(Pts)
                    end
                   end;
           'O':Comb[C]:=Rgn_Or;
           'R':R2[C+1]:=CreateRectRgnIndirect(MakeRect(S));
           'T':if Length(S)<3 then
                raise ECannotTranslate.Create('Syntax error')
               else
                begin
                 ChH:=S[1];
                 ChV:=S[2];
                 Delete(S,1,2);
                 P:=Pos(',',S);
                 if P=0 then
                  raise ECannotTranslate.Create('Invalid number of parameters');
                 X:=GetValue(Copy(S,1,P-1));
                 Delete(S,1,P);
                 P:=Pos(',',S);
                 if P=0 then
                  raise ECannotTranslate.Create('Invalid number of parameters');
                 Y:=GetValue(Copy(S,1,P-1));
                 Delete(S,1,P);
                 R2[C+1]:=0;
                 PE:=ExtRgn;
                 Found:=False;
                 while Assigned(PE) do
                  if CompareText(PE.Name,S)=0 then
                   begin
                    R2[C+1]:=PE.GetRgn;
                    Sz:=PE.RgnSize;
                    Found:=True;
                    Break
                   end
                  else
                   PE:=PE.Next;
                 if not Found then
                  ShowMessage('External region '+S+' not found')
                 else
                  if R2[C+1]<>0 then
                   begin
                    GetRgnBox(R2[C+1],R);
                    if Sz.CX<>-1 then
                     begin
                      R.Left:=0;
                      R.Right:=Sz.CX
                     end;
                    if Sz.CY<>-1 then
                     begin
                      R.Top:=0;
                      R.Bottom:=Sz.CY
                     end;
                    case ChH of
                     'C':OffsetRgn(R2[C+1],X-(R.Right+R.Left) div 2,0);
                     'L':OffsetRgn(R2[C+1],X-R.Left,0);
                     'R':OffsetRgn(R2[C+1],X-R.Right,0)
                    else
                     raise ECannotTranslate.Create('Syntax error')
                    end;
                    case ChV of
                     'B':OffsetRgn(R2[C+1],0,Y-R.Bottom);
                     'C':OffsetRgn(R2[C+1],0,Y-(R.Top+R.Bottom) div 2);
                     'T':OffsetRgn(R2[C+1],0,Y-R.Top)
                    else
                     raise ECannotTranslate.Create('Syntax error')
                    end
                   end
                end;
           'U':begin
                CN:=0;
                for I:=1 to Length(S) do
                 if S[I]=',' then
                  Inc(CN);
                if CN<>5 then
                 raise ECannotTranslate.Create('Invalid number of parameters');
                for I:=1 to 5 do
                 begin
                  P:=Pos(',',S);
                  RR[I]:=GetValue(Copy(S,1,P-1));
                  Delete(S,1,P)
                 end;
                RR[6]:=GetValue(S);
                if Ch='U' then
                 R2[C+1]:=CreateRoundRectRgn(RR[1],RR[2],RR[3],RR[4],RR[5],RR[6])
                else
                 with Owner as TForm do
                  R2[C+1]:=CreateRoundRectRgn(RR[1],RR[2],Width+RR[3],Height+RR[4],RR[5],RR[6])
               end;
           'X':Comb[C]:=Rgn_XOr;
           '{':if C<15 then
                begin
                 Inc(C);
                 Comb[C]:=Rgn_OR;
                 First[C]:=True
                end
               else
                raise ECannotTranslate.Create('Too many brackets');
           '}':if C>1 then
                begin
                 R2[C+1]:=0;
                 Dec(C)
                end
               else
                raise ECannotTranslate.Create('''}'' without ''{''')
          else
           raise ECannotTranslate.Create('Unknown command');
          end;
          if R2[C+1]<>0 then
           if First[C] then
            begin
             R2[C]:=R2[C+1];
             First[C]:=False
            end
           else
            begin
             CombineRgn(R2[C],R2[C],R2[C+1],Comb[C]);
             DeleteObject(R2[C+1])
            end
         end;

       begin
        Result:=True;
        if Rgn<>0 then
         begin
          DeleteObject(Rgn);
          Rgn:=0
         end;
        if Region='' then
         Exit;
        Comb[1]:=Rgn_OR;
        for N:=1 to 16 do
         R2[N]:=0;
        C:=1;
        S:=UpperCase(Region);
        N:=1;
        First[1]:=True;
        try
         P:=Pos(#13#10,S);
         if P<>0 then
          begin
           while P<>0 do
            begin
             ProcessString(Copy(S,1,P-1));
             Delete(S,1,P+1);
             P:=Pos(#13#10,S);
             Inc(N)
            end;
           ProcessString(S)
          end
         else
          ProcessString(S);
         Rgn:=R2[1]
        except
         on E:ECannotTranslate do
          begin
           for P:=1 to 16 do
            if R2[P]<>0 then
             DeleteObject(R2[P]);
           raise ECannotTranslate.Create(E.Message+' in line '+IntToStr(N))
          end
        end
       end;

    function TRegionComp.GetXOrigin;
     begin
      if csDesigning in ComponentState then
       case (Owner as TForm).BorderStyle of
        bsDialog:Result:=SzFrame.X-DFrame.X;
        bsSingle,bsToolWindow:Result:=SzFrame.X-SFrame.X;
        bsNone:Result:=SzFrame.X;
        bsSizeable,bsSizeToolWin:Result:=0
       end
      else
       Result:=0
     end;

    function TRegionComp.GetYOrigin;
     begin
      if csDesigning in ComponentState then
       case (Owner as TForm).BorderStyle of
        bsDialog:Result:=SzFrame.Y-DFrame.Y;
        bsSingle:Result:=SzFrame.Y-SFrame.Y;
        bsNone:Result:=SzFrame.Y+NCapt;
        bsSizeable:Result:=0;
        bsToolWindow:Result:=SzFrame.Y-SFrame.Y+NCapt-SmCapt;
        bsSizeToolWin:Result:=NCapt-SmCapt
       end
      else
       Result:=0
     end;

    function TRegionComp.DesignWidth;
     begin
      Result:=2*GetXOrigin
     end;

    function TRegionComp.DesignHeight;
     begin
      if csDesigning in ComponentState then
       case (Owner as TForm).BorderStyle of
        bsDialog:Result:=2*(SzFrame.Y-DFrame.Y);
        bsSingle:Result:=2*(SzFrame.Y-SFrame.Y);
        bsNone:Result:=2*SzFrame.Y+NCapt;
        bsSizeable:Result:=0;
        bsToolWindow:Result:=2*(SzFrame.Y-SFrame.Y)+NCapt-SmCapt;
        bsSizeToolWin:Result:=NCapt-SmCapt
       end
      else
       Result:=0
     end;

    procedure TRegionComp.CallDefault;
     begin
      Message.Result:=CallWindowProc(OldWndProc,(Owner as TForm).Handle,Message.Msg,Message.wParam,Message.lParam)
     end;

    procedure TRegionComp.Notification;
     begin
      inherited Notification(AComponent,Operation);
      if (AComponent is TExternalRegion) then
       begin
        if not Assigned(ExtRgn) and (Operation=opInsert) then
         ExtRgn:=TExternalRegion(AComponent);
        if (ExtRgn=AComponent) and (Operation=opRemove) then
         ExtRgn:=(AComponent as TExternalRegion).Next
       end
     end;

    procedure TRegionComp.Activate;
     begin
      HookOwner;
      Region:=Region
     end;

    procedure TRegionComp.Deactivate;
     begin
      UnhookOwner;
     end;

    {TFormRegion}

    constructor TFormRegion.Create;
     var I:Integer;
      begin
       with AOwner do
        for I:=0 to ComponentCount-1 do
         if (Components[I] is TFormRegion) and (Components[I]<>Self) then
          raise EDuplicateComponent.Create('Only one TFormRegion component on a form is allowed');
       inherited Create(AOwner);
       FDrawOnDesigning:=True;
       FClientDrag:=False;
       FOnSizeChanged:=nil
      end;

    procedure TFormRegion.HookWndProc;
     var R2:HRgn;
         MBrush:HBrush;
         DC:HDC;
         NewRegion:string;
      begin
       if not Assigned(Owner) then
        Exit;
       case Message.Msg of
        WM_Paint:begin
                  CallDefault(Message);
                  if (csDesigning in ComponentState) and (Rgn<>0) and DrawOnDesigning then
                   begin
                    PostMessage((Owner as TForm).Handle,WM_NCActivate,Integer((Owner as TForm).Active),0);
                    PostMessage((Owner as TForm).Handle,WM_NCPaint,0,0)
                   end
                 end;
        WM_NCPaint:if (csDesigning in ComponentState) and (Rgn<>0) and DrawOnDesigning then
                    with Owner as TForm do
                     begin
                      DC:=GetWindowDC(Handle);
                      SetBkMode(DC,Transparent);
                      R2:=CreateRectRgn(0,0,Width,Height);
                      CombineRgn(R2,R2,Rgn,Rgn_Diff);
                      OffsetRgn(R2,GetXOrigin,GetYOrigin);
                      MBrush:=CreateHatchBrush(hs_DiagCross,clBlack);
                      FillRgn(DC,R2,MBrush);
                      DeleteObject(MBrush);
                      DeleteObject(R2);
                      ReleaseDC(Handle,DC)
                     end;
        WM_Activate:if (csDesigning in ComponentState) and (Rgn<>0) and DrawOnDesigning then
                     PostMessage((Owner as TForm).Handle,WM_NCPaint,0,0);
        WM_Size:with Owner as TForm do
                 begin
                  NewRegion:=Region;
                  if Assigned(FOnSizeChanged) then
                   FOnSizeChanged(Self,NewRegion);
                  Region:=NewRegion
                 end;
        WM_NCHitTest:begin
                      CallDefault(Message);
                      if not (csDesigning in ComponentState) and ClientDrag and (Message.Result=htClient) then
                       Message.Result:=htCaption
                     end
       end;
       if (Message.Msg<>WM_Paint) and (Message.Msg<>WM_NCHitTest) then
        inherited HookWndProc(Message)
      end;

    procedure TFormRegion.SetDrawOnDesigning;
     var R:HRgn;
      begin
       if Value<>FDrawOnDesigning then
        begin
         FDrawOnDesigning:=Value;
         if (csDesigning in ComponentState) and not (csDestroying in Owner.ComponentState) then
          begin
           (Owner as TForm).Invalidate;
           if not Value then
            with Owner as TForm do
             begin
              R:=CreateRectRgnIndirect(BoundsRect);
              SendMessage(Handle,WM_NCPaint,R,0);
              DeleteObject(R)
             end;
          end
        end
      end;

    procedure TFormRegion.RegionCreated;
     var R2:HRgn;
      begin
       if not (csDesigning in ComponentState) and Assigned(Owner) and (Rgn<>0) then
        begin
         R2:=CreateRectRgn(0,0,1,1);
         CombineRgn(R2,Rgn,Rgn,Rgn_Copy);
         SetWindowRgn((Owner as TForm).Handle,R2,(Owner as TForm).Visible)
        end
       else
        if DrawOnDesigning then
         (Owner as TForm).Invalidate
      end;

    procedure TFormRegion.DoDestroy;
     var R:HRgn;
      begin
       if Assigned(Owner) and (csDesigning in ComponentState) and DrawOnDesigning and not (csDestroying in Owner.ComponentState) then
        with Owner as TForm do
         begin
          Invalidate;
          R:=CreateRectRgnIndirect(BoundsRect);
          SendMessage(Handle,WM_NCPaint,R,0);
          DeleteObject(R)
         end
      end;

    {TCaptionLabel}

    constructor TCaptionLabel.Create;
     begin
      inherited Create(Collection);
      FCoords[0]:=0;
      FCoords[1]:=0;
      FCoords[2]:=0;
      FColors[0]:=clInactiveCaptionText;
      FColors[1]:=clCaptionText;
      FVisible:=True;
      FText:='';
      FFont:=TFont.Create;
      FFont.OnChange:=FontChanged
     end;

    destructor TCaptionLabel.Destroy;
     begin
      FFont.Free;
      inherited Destroy
     end;

    function TCaptionLabel.GetDisplayName;
     begin
      if Text='' then
       Result:=inherited GetDisplayName
      else
       Result:=Text
     end;

    procedure TCaptionLabel.Assign;
     begin
      if Source is TCaptionLabel then
       with Source as TCaptionLabel do
        begin
         Self.FColors:=FColors;
         Self.Text:=Text;
         Self.Font:=Font;
         Self.FCoords:=FCoords;
         Self.Visible:=Visible;
        end
      else
       inherited Assign(Source)
     end;

    procedure TCaptionLabel.FontChanged;
     begin
      Changed(False)
     end;

    procedure TCaptionLabel.SetColors;
     begin
      if FColors[Index]<>Value then
       begin
        FColors[Index]:=Value;
        Changed(False)
       end
     end;

    procedure TCaptionLabel.SetText;
     begin
      if FText<>Value then
       begin
        FText:=Value;
        Changed(False)
       end
     end;

    procedure TCaptionLabel.SetFont;
     begin
      FFont.Assign(Value);
      Changed(False)
     end;

    procedure TCaptionLabel.SetCoords;
     begin
      if FCoords[Index]<>Value then
       begin
        FCoords[Index]:=Value;
        Changed(False)
       end
     end;

    procedure TCaptionLabel.SetVisible;
     begin
      if FVisible<>Value then
       begin
        FVisible:=Value;
        Changed(False)
       end
     end;

    procedure TCaptionLabel.Draw;
     var LF:TLogFont;
         NewFont,OldFont:HFont;
      begin
       with LF,Font do
        begin
         lfHeight:=Height;
         lfWidth:=0;
         lfEscapement:=Angle;
         lfOrientation:=Angle;
         if fsBold in Style then
          lfWeight:=FW_Bold
         else
          lfWeight:=FW_Normal;
         lfItalic:=Byte(fsItalic in Style);
         lfUnderline:=Byte(fsUnderline in Style);
         lfStrikeOut:=Byte(fsStrikeOut in Style);
         lfCharSet:=Byte(CharSet);
         lfOutPrecision:=Out_Default_Precis;
         lfClipPrecision:=Clip_Default_Precis;
         lfQuality:=Default_Quality;
         case Pitch of
          fpVariable:lfPitchAndFamily:=Variable_Pitch;
          fpFixed:lfPitchAndFamily:=Fixed_Pitch
         else
          lfPitchAndFamily:=Default_Pitch
         end;
         StrPCopy(lfFaceName,Name)
        end;
       SetBkMode(DC,Transparent);
       NewFont:=CreateFontIndirect(LF);
       OldFont:=SelectObject(DC,NewFont);
       SetTextColor(DC,ColorToRGB(FColors[CIndex]));
       TextOut(DC,Left,Top,PChar(FText),Length(FText));
       SelectObject(DC,OldFont);
       DeleteObject(NewFont)
      end;

    {TCaptionLabels}

    constructor TCaptionLabels.Create;
     begin
      inherited Create(TCaptionLabel);
      CaptionRegion:=ACaptionRegion
     end;

    function TCaptionLabels.Add;
     begin
      Result:=TCaptionLabel(inherited Add)
     end;

    function TCaptionLabels.GetItem;
     begin
      Result:=TCaptionLabel(inherited GetItem(Index))
     end;

    procedure TCaptionLabels.SetItem;
     begin
      inherited SetItem(Index,Value)
     end;

    function TCaptionLabels.GetOwner;
     begin
      Result:=CaptionRegion
     end;

    procedure TCaptionLabels.Update;
     begin
      (CaptionRegion.Owner as TForm).Invalidate
     end;

    procedure TCaptionLabels.Draw;
     var I:Integer;
      begin
       for I:=0 to Count-1 do
        if Items[I].Visible then
         Items[I].Draw(DC,CIndex)
      end;

    {TSysMenuPicture}

    constructor TSysMenuPicture.Create;
     begin
      if not Assigned(AOwner) then
       raise EOwnerNotAssigned.Create('TSysMenuPicture must have a owner');
      inherited Create;
      Owner:=AOwner;
      FGlyph:=TBitmap.Create;
      FSize[0]:=-1;
      FSize[1]:=-1;
      FCoords[0]:=0;
      FCoords[1]:=0;
      FFlags[0]:=True;
      FFlags[1]:=False
     end;

    destructor TSysMenuPicture.Destroy;
     begin
      FGlyph.Free;
      inherited Destroy
     end;

    procedure TSysMenuPicture.Assign;
     begin
      if Source is TSysMenuPicture then
       with Source as TSysMenuPicture do
        begin
         Self.FGlyph.Assign(FGlyph);
         Self.FCoords:=FCoords;
         Self.FFlags:=FFlags;
         Self.FSize:=FSize
        end;
      inherited Assign(Source)
     end;

    procedure TSysMenuPicture.SetGlyph;
     begin
      FGlyph.Assign(Value);
      FGlyph.TransparentMode:=tmAuto;
      FGlyph.Transparent:=Transparent;
      Redraw
     end;

    procedure TSysMenuPicture.SetCoords;
     begin
      if Value<>FCoords[Index] then
       begin
        FCoords[Index]:=Value;
        Redraw
       end
     end;

    procedure TSysMenuPicture.SetFlags;
     begin
      if Value<>FFlags[Index] then
       begin
        FFlags[Index]:=Value;
        if Index=0 then
         begin
          FGlyph.TransparentMode:=tmAuto;
          FGlyph.Transparent:=Value
         end;
        Redraw
       end
     end;

    procedure TSysMenuPicture.SetSize;
     begin
      if Value<>FSize[Index] then
       begin
        FSize[Index]:=Value;
        if FGlyph.Empty then
         Redraw
       end
     end;

    procedure TSysMenuPicture.Redraw;
     begin
      if Visible then
       (Owner.Owner as TForm).Invalidate
     end;

    procedure TSysMenuPicture.Draw;
     var Canvas:TCanvas;
         Icon:HIcon;
         W,H,DX,DY:Integer;
         R:TRect;
      begin
       if Visible then
        begin
         DX:=Left;
         DY:=Top;
         if not FGlyph.Empty then
          begin
           Canvas:=TCanvas.Create;
           Canvas.Handle:=DC;
           Canvas.Draw(DX,DY,FGlyph);
           Canvas.Handle:=0;
           Canvas.Free
          end
         else
          begin
           W:=Width;
           H:=Height;
           if W<=0 then
            W:=GetSystemMetrics(SM_CXSmIcon);
           if H<=0 then
            H:=GetSystemMetrics(SM_CYSmIcon);
           if csDesigning in Owner.ComponentState then
            begin
             R:=Rect(0,0,W,H);
             OffsetRect(R,DX,DY);
             SelectObject(DC,GetStockObject(White_Brush));
             Rectangle(DC,R.Left,R.Top,R.Right,R.Bottom)
             //DrawFocusRect(DC,R)
            end
           else
            begin
             Icon:=LoadImage(HInstance,'MAINICON',Image_Icon,W,H,0);
             DrawIconEx(DC,DX,DY,Icon,W,H,0,0,DI_Normal);
             DeleteObject(Icon)
            end
          end
        end
      end;

    function TSysMenuPicture.RealWidth;
     begin
      if FGlyph.Empty then
       if Width>0 then
        Result:=Width
       else
        Result:=GetSystemMetrics(SM_CXSmIcon)
      else
       Result:=FGlyph.Width
     end;

    function TSysMenuPicture.RealHeight;
     begin
      if FGlyph.Empty then
       if Height>0 then
        Result:=Height
       else
        Result:=GetSystemMetrics(SM_CYSmIcon)
      else
       Result:=FGlyph.Height
     end;

    {TCaptionRegion}

    constructor TCaptionRegion.Create;
     var I:Integer;
      begin
       with AOwner as TForm do
        for I:=0 to ComponentCount-1 do
         if (Components[I] is TCaptionRegion) and (Components[I]<>Self) then
          raise EDuplicateComponent.Create('Only one TCaptionRegion component on a form is allowed');
       inherited Create(AOwner);
       FColors[1]:=clActiveCaption;
       FColors[0]:=clInactiveCaption;
       FOnSizeChanged:=nil;
       FSysMenuPicture:=TSysMenuPicture.Create(Self);
       FLabels:=TCaptionLabels.Create(Self);
       ClickTime:=0
      end;

    procedure TCaptionRegion.RegionCreated;
     begin
      (Owner as TForm).Invalidate
     end;

    procedure TCaptionRegion.DoDestroy;
     var R:HRgn;
      begin
       if Assigned(Owner) and not (csDestroying in Owner.ComponentState) then
        with Owner as TForm do
         begin
          Invalidate;
          R:=CreateRectRgnIndirect(BoundsRect);
          SendMessage(Handle,WM_NCPaint,R,0);
          DeleteObject(R)
         end;
       FLabels.Free;
       FSysMenuPicture.Free
      end;

    procedure TCaptionRegion.Loaded;
     begin
      inherited Loaded;
      with FSysMenuPicture.FGlyph do
       if not Empty then
        begin
         TransparentMode:=tmAuto;
         Transparent:=FSysMenuPicture.Transparent
        end
     end;

    procedure TCaptionRegion.SetColor;
     begin
      if FColors[Index]<>Value then
       begin
        FColors[Index]:=Value;
        (Owner as TForm).Invalidate
       end
     end;

    procedure TCaptionRegion.SetSysMenuPicture;
     begin
      FSysMenuPicture.Assign(Value);
      (Owner as TForm).Invalidate
     end;

    procedure TCaptionRegion.SetLabels;
     begin
      FLabels.Assign(Value);
      (Owner as TForm).Invalidate
     end;

    procedure TCaptionRegion.HookWndProc;
     var R2:HRgn;
         N,RRes:Integer;
         MBrush:HBrush;
         Pt:TPoint;
         DC:HDC;
         PS:TPaintStruct;
         NewRegion:string;
         R:TRect;
      begin
       case Message.Msg of
        WM_Paint:begin
                  N:=Integer((Owner as TForm).Active);
                  if (Message.WParam=0) and (Rgn<>0) and (FColors[N]<>clNone) then
                   with Owner as TForm do
                    begin
                     DC:=BeginPaint(Handle,PS);
                     SetBkMode(DC,Transparent);
                     R2:=CreateRectRgn(0,0,1,1);
                     CombineRgn(R2,Rgn,Rgn,Rgn_Copy);
                     OffsetRgn(R2,Left-ClientOrigin.X+GetXOrigin,Top-ClientOrigin.Y+GetYOrigin);
                     MBrush:=CreateSolidBrush(ColorToRGB(FColors[N]));
                     FillRgn(DC,R2,MBrush);
                     if FSysMenuPicture.Visible then
                      FSysMenuPicture.Draw(DC);
                     FLabels.Draw(DC,N);
                     Message.Result:=SendMessage(Handle,WM_Paint,DC,0);
                     DeleteObject(MBrush);
                     DeleteObject(R2);
                     EndPaint(Handle,PS)
                    end
                  else
                   CallDefault(Message)
                 end;
        WM_SysChar:if not (csDesigning in ComponentState) and (Message.WParam=VK_Space) and FSysMenuPicture.Visible and Assigned(FPopupMenu) then
                    begin
                     Message.Result:=0;
                     with (Self.Owner as TForm),FSysMenuPicture do
                      FPopupMenu.Popup(FSysMenuPicture.Left+ClientOrigin.X,FSysMenuPicture.Top+RealHeight+ClientOrigin.Y)
                    end
                   else
                    CallDefault(Message);
        WM_KeyDown,WM_SysKeyDown:if not (csDesigning in ComponentState) and Assigned(FPopupMenu) then
                                  Message.Result:=Integer(FPopupMenu.IsShortCut(TWMKey(Message)))
                                 else
                                  CallDefault(Message);
        WM_Activate:begin
                     CallDefault(Message);
                     N:=Integer((Owner as TForm).Active);
                     if (Rgn<>0) and (FColors[N]<>clNone) then
                      begin
                       R2:=CreateRectRgn(0,0,1,1);
                       CombineRgn(R2,Rgn,Rgn,Rgn_Copy);
                       with Owner as TForm do
                        OffsetRgn(R2,Left-ClientOrigin.X+GetXOrigin,Top-ClientOrigin.Y+GetYOrigin);
                       InvalidateRgn((Owner as TForm).Handle,R2,True);
                       SendMessage((Owner as TForm).Handle,WM_NCPaint,R2,0);
                       DeleteObject(R2)
                      end
                    end;
        WM_Size:with Owner as TForm do
                 begin
                  NewRegion:=Region;
                  if Assigned(FOnSizeChanged) then
                   FOnSizeChanged(Self,NewRegion);
                  Region:=NewRegion;
                  CallDefault(Message)
                 end;
        WM_LButtonDown:begin
                        if not (csDesigning in ComponentState) and Assigned(FPopupMenu) then
                         begin
                          Pt:=SmallPointToPoint(TSmallPoint(Message.LParam));
                          with FSysMenuPicture do
                           if PtInRect(Rect(Left,Top,Left+RealWidth,Top+RealHeight),Pt) then
                            begin
                             if (GetTickCount-ClickTime)>GetDoubleClickTime then
                              with Self.Owner as TForm do
                               FPopupMenu.Popup(FSysMenuPicture.Left+ClientOrigin.X,FSysMenuPicture.Top+RealHeight+ClientOrigin.Y);
                             ClickTime:=GetTickCount
                            end
                           else
                            CallDefault(Message)
                         end
                        else
                         CallDefault(Message)
                       end;
        WM_NCRButtonDown:if (Message.WParam=HTCaption) and Assigned(FPopupMenu) then
                          begin
                           FPopupMenu.Popup(Message.LParamLo,Message.LParamHi);
                           Message.Result:=0
                          end
                         else
                          CallDefault(Message);
        WM_LButtonDblClk:begin
                          if not (csDesigning in ComponentState) and Assigned(FPopupMenu) then
                           begin
                            Pt:=SmallPointToPoint(TSmallPoint(Message.LParam));
                            with FSysMenuPicture do
                             if PtInRect(Rect(Left,Top,Left+RealWidth,Top+RealHeight),Pt) then
                              for N:=0 to FPopupMenu.Items.Count-1 do
                               if FPopupMenu.Items[N].Default then
                                begin
                                 FPopupMenu.Items[N].Click;
                                 Break
                                end
                           end;
                          CallDefault(Message)
                         end;
        WM_NCLButtonDblClk:with Owner as TForm do
                            if (BorderStyle=bsNone) and (biMaximize in BorderIcons) then
                             if Message.WParam=HTCaption then
                              begin
                               if WindowState=wsMaximized then
                                WindowState:=wsNormal
                               else
                                WindowState:=wsMaximized;
                               Message.Result:=0
                              end
                             else
                              begin
                              end
                            else
                             CallDefault(Message);
        WM_NCHitTest:begin
                      CallDefault(Message);
                      with Owner as TForm do
                       if (Message.Result<>HTCaption) and not (csDesigning in ComponentState) then
                        begin
                         Pt:=SmallPointToPoint(TSmallPoint(Message.LParam));
                         with FSysMenuPicture do
                          begin
                           R:=Rect(0,0,RealWidth,RealHeight);
                           OffsetRect(R,Left+ClientOrigin.X,Top+ClientOrigin.Y)
                          end;
                         if PtInRegion(Rgn,Pt.X-Left,Pt.Y-Top) and not (FSysMenuPicture.Visible and PtInRect(R,Pt)) then
                          Message.Result:=HTCaption
                        end
                     end
       else
        inherited HookWndProc(Message)
       end
      end;

    {TExternalRegion}

    constructor TExternalRegion.Create;
     begin
      if not (AOwner is TForm) then
       raise EOwnerMustBeForm.Create('The owner of '+ClassName+' component must be the TForm descendant');
      inherited Create(AOwner);
      First:=True;
      Next:=nil;
      FOwnRegion:=True;
      Rgn:=0;
      FOldName:=''
     end;

    destructor TExternalRegion.Destroy;
     begin
      if Rgn<>0 then
       DeleteObject(Rgn);
      inherited Destroy
     end;

    procedure TExternalRegion.Notification;
     begin
      inherited Notification(AComponent,Operation);
      if Operation=opInsert then
       begin
        if (AComponent is TRegionComp) and First then
         (AComponent as TRegionComp).ExtRgn:=Self;
        if (AComponent is TExternalRegion) and (AComponent<>Self) and not Assigned(Next) then
         begin
          Next:=TExternalRegion(AComponent);
          (AComponent as TExternalRegion).First:=False
         end
       end
      else
       if (AComponent is TExternalRegion) then
        if AComponent=Next then
         Next:=(AComponent as TExternalRegion).Next
        else
         if (AComponent as TExternalRegion).First and ((AComponent as TExternalRegion).Next=Self) then
          First:=True
     end;

    procedure TExternalRegion.RgnChanged;
     begin
      if Rgn<>0 then
       DeleteObject(Rgn);
      Rgn:=0;
      if Owner is TForm then
       SendMessage((Owner as TForm).Handle,WM_ExtRgnChanged,0,LongInt(Self))
     end;

    procedure TExternalRegion.SetName;
     begin
      FOldName:=Name;
      inherited SetName(Value);
      if Owner is TForm then
       SendMessage((Owner as TForm).Handle,WM_ExtRgnNewName,0,LongInt(Self));
      FOldName:=''
     end;

    function TExternalRegion.GetRgn;
     begin
      if Rgn<>0 then
       begin
        Result:=CreateRectRgn(0,0,1,1);
        CombineRgn(Result,Rgn,Rgn,Rgn_Copy)
       end
      else
       if FOwnRegion then
        begin
         Rgn:=MakeRgn;
         Result:=CreateRectRgn(0,0,1,1);
         CombineRgn(Result,Rgn,Rgn,Rgn_Copy)
        end
       else
        Result:=MakeRgn
     end;

    function TExternalRegion.RgnSize;
     begin
      Result.CX:=-1;
      Result.CY:=-1
     end;

    {TTextExtRgn}

    constructor TTextExtRgn.Create;
     begin
      inherited Create(AOwner);
      FFont:=TFont.Create;
      FFont.OnChange:=FontChanged;
      FAngle:=0;
      FText:='';
      FWeight:=-1;
      FTransparent:=True
     end;

    destructor TTextExtRgn.Destroy;
     begin
      FFont.Free;
      inherited Destroy
     end;

    procedure TTextExtRgn.SetText;
     begin
      if Value<>FText then
       begin
        FText:=Value;
        RgnChanged
       end
     end;

    procedure TTextExtRgn.SetAngle;
     begin
      if Value<>FAngle then
       begin
        FAngle:=Value;
        RgnChanged
       end
     end;

    procedure TTextExtRgn.SetWeight;
     begin
      if Value<>FWeight then
       begin
        FWeight:=Value;
        RgnChanged
       end
     end;

    procedure TTextExtRgn.SetTransparent;
     begin
      if Value<>FTransparent then
       begin
        FTransparent:=Value;
        RgnChanged
       end
     end;

    procedure TTextExtRgn.FontChanged;
     begin
      RgnChanged
     end;

    procedure TTextExtRgn.SetFont;
     begin
      FFont.Assign(Value);
      RgnChanged
     end;

    function TTextExtRgn.MakeRgn;
     var DC:HDC;
         LF:TLogFont;
         NewFont,OldFont:HFont;
      begin
       with LF,FFont do
        begin
         lfHeight:=Height;
         lfWidth:=0;
         lfEscapement:=FAngle;
         lfOrientation:=FAngle;
         if FWeight<0 then
          if fsBold in Style then
           lfWeight:=FW_Bold
          else
           lfWeight:=FW_Normal
         else
          lfWeight:=FWeight;
         lfItalic:=Byte(fsItalic in Style);
         lfUnderline:=Byte(fsUnderline in Style);
         lfStrikeOut:=Byte(fsStrikeOut in Style);
         lfCharSet:=Byte(CharSet);
         lfOutPrecision:=Out_Default_Precis;
         lfClipPrecision:=Clip_Default_Precis;
         lfQuality:=Default_Quality;
         case Pitch of
          fpVariable:lfPitchAndFamily:=Variable_Pitch;
          fpFixed:lfPitchAndFamily:=Fixed_Pitch
         else
          lfPitchAndFamily:=Default_Pitch
         end;
         StrPCopy(lfFaceName,Name)
        end;
       NewFont:=CreateFontIndirect(LF);
       DC:=CreateDC('DISPLAY',nil,nil,nil);
       OldFont:=SelectObject(DC,NewFont);
       BeginPath(DC);
       if FTransparent then
        SetBkMode(DC,Windows.Transparent);
       TextOut(DC,0,0,PChar(FText),Length(FText));
       EndPath(DC);
       Result:=PathToRegion(DC);
       SelectObject(DC,OldFont);
       DeleteObject(NewFont);
       DeleteDC(DC)
      end;

    {TPictExtRgn}

    constructor TPictExtRgn.Create;
     begin
      inherited Create(AOwner);
      FGlyph:=TBitmap.Create;
      FTransColor:=clOlive;
      FInverted:=False;
      FAlign[0]:=raResult;
      FAlign[1]:=raResult
     end;

    destructor TPictExtRgn.Destroy;
     begin
      FGlyph.Free;
      inherited Destroy
     end;

    procedure TPictExtRgn.SetGlyph;
     begin
      FGlyph.Assign(Value);
      RgnChanged
     end;

    procedure TPictExtRgn.SetTransColor;
     begin
      if Value<>FTransColor then
       begin
        FTransColor:=Value;
        RgnChanged
       end
     end;

    procedure TPictExtRgn.SetInverted;
     begin
      if FInverted<>Value then
       begin
        FInverted:=Value;
        RgnChanged
       end
     end;

    procedure TPictExtRgn.SetAlign;
     begin
      if Value<>FAlign[Index] then
       begin
        FAlign[Index]:=Value;
        if not FGlyph.Empty then
         RgnChanged
       end
     end;

    function TPictExtRgn.RgnSize;
     begin
      if not FGlyph.Empty and (FAlign[0]=raSource) then
       Result.CX:=FGlyph.Width
      else
       Result.CX:=-1;
      if not FGlyph.Empty and (FAlign[1]=raSource) then
       Result.CY:=FGlyph.Height
      else
       Result.CY:=-1
     end;

    function TPictExtRgn.MakeRgn;
     begin
      if not FGlyph.Empty then
       Result:=BitmapToRgn(FGlyph)
      else
       Result:=0
     end;

    function TPictExtRgn.BitmapToRgn;
     const AllocUnit=100;
      var BMP:TBitmap;
          MaxRects:Integer;
          HData:HGlobal;
          PData:PRgnData;
          CB,CR,CG,LR,LG,LB:Byte;
          P32:Pointer;
          X,X0,Y:Integer;
          P:PLongInt;
          PR:PRect;
          H:Hrgn;
       begin
        Result:=0;
        BMP:=TBitmap.Create;
        BMP.Assign(Bitmap);
        BMP.HandleType:=bmDIB;
        BMP.PixelFormat:=pf32bit;
        MaxRects:=AllocUnit;
        HData:=GlobalAlloc(GMem_Moveable,SizeOf(TRgnDataHeader)+SizeOf(TRect)*MaxRects);
        PData := GlobalLock(HData);
        PData^.RDH.dwSize:=SizeOf(TRgnDataHeader);
        PData^.RDH.iType:=RDH_Rectangles;
        PData^.RDH.nCount:=0;
        PData^.RDH.nRgnSize:=0;
        SetRect(PData^.RDH.rcBound,MaxInt,MaxInt,0,0);
        LR:=GetRValue(ColorToRGB(TransColor));
        LG:=GetGValue(ColorToRGB(TransColor));
        LB:=GetBValue(ColorToRGB(TransColor));
        for Y:=0 to Bitmap.Height-1 do
         begin
          X:=-1;
          P32:=BMP.ScanLine[Y];
          while X+1<Bitmap.Width do
           begin
            Inc(X);
            X0:=X;
            P:=PLongInt(P32);
            Inc(PChar(P),X*SizeOf(LongInt));
            while X<Bitmap.Width do
             begin
              CR:=GetBValue(P^);
              CG:=GetGValue(P^);
              CB:=GetRValue(P^);
              if ((CR=LR) and (CG=LG) and (CB=LB)) xor FInverted then
               Break;
              Inc(PChar(P),SizeOf(LongInt));
              Inc(X)
             end;
            if X>X0 then
             begin
              if PData^.RDH.nCount>=MaxRects then
               begin
                GlobalUnlock(HData);
                Inc(MaxRects,AllocUnit);
                HData:=GlobalReAlloc(HData,SizeOf(TRgnDataHeader)+SizeOf(TRect)*MaxRects,GMem_Moveable);
                PData:=GlobalLock(HData)
               end;
              PR:=@PData^.Buffer[PData^.RDH.nCount*SizeOf(TRect)];
              SetRect(PR^,X0,Y,X,Y+1);
              if X0<PData^.RDH.rcBound.Left then
               PData^.RDH.rcBound.Left:=X0;
              if Y<PData^.RDH.rcBound.Top then
               PData^.RDH.rcBound.Top:=Y;
              if X>PData^.RDH.rcBound.Right then
               PData^.RDH.rcBound.Left:=X;
              if Y+1>PData^.RDH.rcBound.Bottom then
               PData^.RDH.rcBound.Bottom:=Y+1;
              Inc(PData^.RDH.nCount);
              if PData^.RDH.nCount=2000 then
               begin
                H:=ExtCreateRegion(nil,SizeOf(TRgnDataHeader)+(SizeOf(TRect)*MaxRects),PData^);
                if Result<>0 then
                 begin
                  CombineRgn(Result,Result,H,RGN_OR);
                  DeleteObject(H);
                 end
                else
                 Result:=H;
                PData^.RDH.nCount:=0;
                SetRect(PData^.RDH.rcBound,MaxInt,MaxInt,0,0)
               end
             end
           end
         end;
        H:=ExtCreateRegion(nil,SizeOf(TRgnDataHeader)+(SizeOf(TRect)*MaxRects),PData^);
        if Result<>0 then
         begin
          CombineRgn(Result,Result,H,RGN_OR);
          DeleteObject(H);
         end
        else
         Result:=H;
        GlobalFree(HData);
        BMP.Free
       end;

    {THandlingControl}

    constructor THandlingControl.Create;
     begin
      inherited Create(AOwner);
      if not (AOwner is TForm) then
       raise EOwnerMustBeForm.Create('The owner of '+ClassName+' component must be the TForm descendant');
      HookOwner
     end;

    destructor THandlingControl.Destroy;
     begin
      UnhookOwner;
      inherited Destroy
     end;

    procedure THandlingControl.HookOwner;
     begin
      if not Assigned(Owner) then
       Exit;
      OldWndProc:=TFarProc(GetWindowLong((Owner as TForm).Handle,GWL_WndProc));
      NewWndProc:=MakeObjectInstance(HookWndProc);
      SetWindowLong((Owner as TForm).Handle,GWL_WndProc,LongInt(NewWndProc))
     end;

    procedure THandlingControl.UnhookOwner;
     begin
      if Assigned(Owner) and Assigned(OldWndProc) and not (csDestroying in (Owner as TForm).ComponentState) then
       SetWindowLong((Owner as TForm).Handle,GWL_WndProc,LongInt(OldWndProc));
      if Assigned(NewWndProc) then
       FreeObjectInstance(NewWndProc);
      NewWndProc:=nil;
      OldWndProc:=nil
     end;

    procedure THandlingControl.HookWndProc;
     begin
      CallDefault(Message)
     end;

    procedure THandlingControl.CallDefault;
     begin
      Message.Result:=CallWindowProc(OldWndProc,(Owner as TForm).Handle,Message.Msg,Message.wParam,Message.lParam)
     end;

    procedure THandlingControl.Activate;
     begin
      HookOwner
     end;

    procedure THandlingControl.Deactivate;
     begin
      UnhookOwner
     end;

    {TMenuHotSpot}

    constructor TMenuHotSpot.Create;
     begin
      inherited Create(AOwner);
      FGlyph:=TBitmap.Create;
      Width:=16;
      Height:=16;
      FColors[0]:=clInactiveCaption;
      FColors[1]:=clActiveCaption;
      FTransparent:=True;
      ClickTime:=0
     end;

    destructor TMenuHotSpot.Destroy;
     begin
      FGlyph.Free;
      inherited Destroy
     end;

    procedure TMenuHotSpot.SetGlyph;
     begin
      FGlyph.Assign(Value);
      if not FGlyph.Empty then
       begin
        Width:=FGlyph.Width;
        Height:=FGlyph.Height;
        FGlyph.Transparent:=FTransparent;
        if FTransparent then
         FGlyph.TransparentMode:=tmAuto
       end;
      Invalidate
     end;

    procedure TMenuHotSpot.SetTransparent;
     begin
      if Value<>FTransparent then
       begin
        FTransparent:=Value;
        if not FGlyph.Empty then
         begin
          FGlyph.TransparentMode:=tmAuto;
          FGlyph.Transparent:=Value
         end;
        Refresh
       end
     end;

    procedure TMenuHotSpot.SetColor;
     begin
      if Value<>FColors[Index] then
       begin
        FColors[Index]:=Value;
        if Transparent then
         Invalidate
       end
     end;

    procedure TMenuHotSpot.MouseDown;
     var Pt:TPoint;
      begin
       if (Button=mbLeft) and Assigned(PopupMenu) then
        begin
         if (GetTickCount-ClickTime)>GetDoubleClickTime then
          begin
           Pt:=(Owner as TControl).ClientToScreen(Point(Left,Top+Height));
           PopupMenu.Popup(Pt.X,Pt.Y)
          end;
         ClickTime:=GetTickCount
        end
      end;

    procedure TMenuHotSpot.DblClick;
     var I:Integer;
      begin
       inherited DblClick;
       if Assigned(PopupMenu) then
        for I:=0 to PopupMenu.Items.Count-1 do
         if PopupMenu.Items[I].Default then
          begin
           PopupMenu.Items[I].Click;
           Break
          end
      end;

    procedure TMenuHotSpot.Paint;
     var Icon:HIcon;
      begin
       Canvas.Brush.Style:=bsSolid;
       if (Owner as TForm).Active then
        Canvas.Brush.Color:=FColors[1]
       else
        Canvas.Brush.Color:=FColors[0];
       Canvas.FillRect(ClientRect);
       if FGlyph.Empty then
        begin
         Width:=16;
         Height:=16;
         Icon:=LoadImage(HInstance,'MAINICON',Image_Icon,16,16,0);
         DrawIconEx(Canvas.Handle,0,0,Icon,16,16,0,0,DI_Normal);
         DeleteObject(Icon);
        end
       else
        Canvas.Draw(0,0,FGlyph)
      end;

    procedure TMenuHotSpot.HookWndProc;
     begin
      CallDefault(Message);
      if Message.Msg=WM_Activate then
       Refresh
     end;

    {TSystemButton}

    constructor TSystemButton.Create;
     begin
      inherited Create(AOwner);
      FKind:=sbClose;
      FAutoSize:=True;
      FOnDrawButton:=nil;
      FPressed:=False;
      FPressLoop:=False;
      FGlyph:=TBitmap.Create;
      inherited SetBounds(Left,Top,GetSystemMetrics(SM_CXSize)-2,GetSystemMetrics(SM_CYSize)-4)
     end;

    destructor TSystemButton.Destroy;
     begin
      FGlyph.Free;
      inherited Destroy
     end;

    procedure TSystemButton.SetKind;
     begin
      if Value<>FKind then
       begin
        FKind:=Value;
        Invalidate
       end
     end;

    procedure TSystemButton.SetAutoSize;
     begin
      if Value<>FAutoSize then
       begin
        FAutoSize:=Value;
        if Value then
         inherited SetBounds(Left,Top,GetSystemMetrics(SM_CXSize)-2,GetSystemMetrics(SM_CYSize)-4)
       end
     end;

    procedure TSystemButton.SetGlyph;
     begin
      FGlyph.Assign(Value);
      if not FGlyph.Empty then
       FKind:=sbGlyph;
      if FKind=sbGlyph then
       Invalidate
     end;

    procedure TSystemButton.Paint;
     var BState:Integer;
         R:TRect;
      begin
       if FKind<>sbOwnerDraw then
        begin
         R:=Rect(0,0,Width,Height);
         if not Enabled then
          BState:=DFCS_Inactive
         else
          if Pressed then
           BState:=DFCS_Pushed
          else
           BState:=0;
         case FKind of
          sbClose:DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionClose or BState);
          sbMinimize:if (Owner as TForm).WindowState=wsMinimized then
                      DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionRestore or BState)
                     else
                      DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionMin or BState);
          sbMaximize:if (Owner as TForm).WindowState=wsMaximized then
                      DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionRestore or BState)
                     else
                      DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionMax or BState);
          sbHelp:DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionHelp or BState);
          sbGlyph:begin
                   DrawFrameControl(Canvas.Handle,R,DFC_Button,DFCS_ButtonPush or BState);
                   if not FGlyph.Empty then
                    Canvas.Draw((Width-FGlyph.Width) div 2,(Height-FGlyph.Height) div 2,FGlyph)
                  end;
          sbOwnerDrawFace:begin
                           DrawFrameControl(Canvas.Handle,R,DFC_Button,DFCS_ButtonPush or BState);
                           if Assigned(FOnDrawButton) then
                            FOnDrawButton(Self)
                          end
         end
        end
       else
        if Assigned(FOnDrawButton) then
         FOnDrawButton(Self)
      end;

    procedure TSystemButton.SetBounds;
     begin
      if (AWidth<>Width) or (AHeight<>Height) then
       FAutoSize:=False;
      inherited SetBounds(ALeft,ATop,AWidth,AHeight)
     end;

    procedure TSystemButton.CMEnabledChanged;
     begin
      inherited
      Refresh
     end;

    procedure TSystemButton.MouseDown;
     begin
      inherited MouseDown(Button,Shift,X,Y);
      if Button=mbLeft then
       begin
        FPressed:=True;
        Refresh;
        FPressLoop:=True
       end
     end;

    procedure TSystemButton.MouseMove;
     begin
      inherited MouseMove(Shift,X,Y);
      if FPressLoop then
       if (X>=0) and (X<ClientWidth) and (Y>=0) and (Y<ClientWidth) then
        begin
         if not FPressed then
          begin
           FPressed:=True;
           Refresh
          end
        end
       else
        if FPressed then
         begin
          FPressed:=False;
          Refresh
         end
     end;

    procedure TSystemButton.MouseUp;
     begin
      inherited MouseUp(Button,Shift,X,Y);
      if FPressLoop and (Button=mbLeft) then
       begin
        FPressLoop:=False;
        FPressed:=False;
        Refresh;
        if  (X>=0) and (X<ClientWidth) and (Y>=0) and (Y<ClientWidth) then
         case FKind of
          sbClose:SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_Close,-1);
          sbMinimize:if (Owner as TForm).WindowState=wsMinimized then
                      SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_Restore,-1)
                     else
                      SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_Minimize,-1);
          sbMaximize:if (Owner as TForm).WindowState=wsMaximized then
                      SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_Restore,-1)
                     else
                      SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_Maximize,-1);
          sbHelp:SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_ContextHelp,-1)
         end
       end
     end;

    procedure TSystemButton.Loaded;
     begin
      inherited Loaded;
      if AutoSize then
       inherited SetBounds(Left,Top,GetSystemMetrics(SM_CXSize)-2,GetSystemMetrics(SM_CYSize)-4)
     end;

    procedure TSystemButton.HookWndProc;
     begin
      CallDefault(Message);
      if (Message.Msg=WM_Size) and (FKind in [sbMinimize,sbMaximize,sbOwnerDraw,sbOwnerDrawFace]) then
       Refresh
     end;

    {TCaptionText}

    constructor TCaptionText.Create;
     begin
      if not (AOwner is TForm) then
       raise EOwnerMustBeForm.Create('The owner of TCaptionText component must be TForm descendant');
      inherited Create(AOwner);
      FColors[0]:=clInactiveCaptionText;
      FColors[1]:=clCaptionText;
      FAutoSize:=True;
      FAngle:=0;
      HookOwner
     end;

    destructor TCaptionText.Destroy;
     begin
      UnhookOwner;
      inherited Destroy
     end;

    procedure TCaptionText.HookOwner;
     begin
      if not Assigned(Owner) then
       Exit;
      OldWndProc:=TFarProc(GetWindowLong((Owner as TForm).Handle,GWL_WndProc));
      NewWndProc:=MakeObjectInstance(HookWndProc);
      SetWindowLong((Owner as TForm).Handle,GWL_WndProc,LongInt(NewWndProc))
     end;

    procedure TCaptionText.UnhookOwner;
     begin
      if Assigned(Owner) and Assigned(OldWndProc) and not (csDestroying in (Owner as TForm).ComponentState) then
       SetWindowLong((Owner as TForm).Handle,GWL_WndProc,LongInt(OldWndProc));
      if Assigned(NewWndProc) then
       FreeObjectInstance(NewWndProc);
      NewWndProc:=nil;
      OldWndProc:=nil
     end;

    procedure TCaptionText.CallDefault;
     begin
      Message.Result:=CallWindowProc(OldWndProc,(Owner as TForm).Handle,Message.Msg,Message.wParam,Message.lParam)
     end;

    procedure TCaptionText.HookWndProc;
     begin
      CallDefault(Message);
      if Message.Msg=WM_Activate then
       Refresh
     end;

    procedure TCaptionText.SetAngle;
     begin
      if Value<>FAngle then
       begin
        while Value<0 do
         Inc(Value,3600);
        while Value>3599 do
         Dec(Value,3600);
        FAngle:=Value;
        if FAutoSize then
         SetSize;
        Invalidate
       end
     end;

    procedure TCaptionText.SetAutoSize;
     begin
      if Value<>FAutoSize then
       begin
        FAutoSize:=Value;
        if Value then
         SetSize
       end
      end;

    procedure TCaptionText.SetColor;
     begin
      if FColors[Index]<>Value then
       begin
        FColors[Index]:=Value;
        Invalidate
       end
     end;

    procedure TCaptionText.CMTextChanged;
     begin
      inherited;
      if FAutoSize then
       SetSize;
      Invalidate
     end;

    procedure TCaptionText.CMFontChanged;
     begin
      inherited;
      if AutoSize then
       SetSize
     end;

    function TCaptionText.CalcSize;
     var DC:HDC;
         Sz:TSize;
         OldFont:HFont;
         RAngle:Double;
      begin
       DC:=CreateDC('DISPLAY',nil,nil,nil);
       OldFont:=SelectObject(DC,Font.Handle);
       GetTextExtentPoint32(DC,PChar(Caption),Length(Caption),Sz);
       if FAngle=0 then
        Result:=Sz
       else
        begin
         RAngle:=FAngle/1800*Pi;
         Result.CX:=Round(Abs(Sz.CX*Cos(RAngle))+Abs(Sz.CY*Sin(RAngle)));
         Result.CY:=Round(Abs(Sz.CX*Sin(RAngle))+Abs(Sz.CY*Cos(RAngle)))
        end;
       SelectObject(DC,OldFont);
       DeleteDC(DC)
      end;

    procedure TCaptionText.SetSize;
     var Sz:TSize;
      begin
       Sz:=CalcSize;
       Width:=Sz.CX;
       Height:=Sz.CY
      end;

    procedure TCaptionText.Paint;
     var LF:TLogFont;
         Sz:TSize;
         X,Y:Integer;
         NewFont,OldFont:HFont;
         SA,CA:Extended;
      begin
       with LF,Font do
        begin
         lfHeight:=Height;
         lfWidth:=0;
         lfEscapement:=FAngle;
         lfOrientation:=FAngle;
         if fsBold in Style then
          lfWeight:=FW_Bold
         else
          lfWeight:=FW_Normal;
         lfItalic:=Byte(fsItalic in Style);
         lfUnderline:=Byte(fsUnderline in Style);
         lfStrikeOut:=Byte(fsStrikeOut in Style);
         lfCharSet:=Byte(CharSet);
         lfOutPrecision:=Out_Default_Precis;
         lfClipPrecision:=Clip_Default_Precis;
         lfQuality:=Default_Quality;
         case Pitch of
          fpVariable:lfPitchAndFamily:=Variable_Pitch;
          fpFixed:lfPitchAndFamily:=Fixed_Pitch
         else
          lfPitchAndFamily:=Default_Pitch
         end;
         StrPCopy(lfFaceName,Name)
        end;
       SetBkMode(Canvas.Handle,Transparent);
       NewFont:=CreateFontIndirect(LF);
       OldFont:=SelectObject(Canvas.Handle,NewFont);
       GetTextExtentPoint32(Canvas.Handle,PChar(Caption),Length(Caption),Sz);
       SinCos(FAngle/1800*Pi,SA,CA);
       case FAngle of
        0:begin
           X:=0;
           Y:=0
          end;
        1..899:begin
                X:=0;
                Y:=Round(Sz.CX*SA)
               end;
        900:begin
             X:=0;
             Y:=Sz.CX
            end;
        901..1799:begin
                   X:=Round(Abs(Sz.CX*CA));
                   Y:=Round(Sz.CX*SA+Abs(Sz.CY*CA))
                  end;
        1800:begin
              X:=Sz.CX;
              Y:=Sz.CY
             end;
        1801..2699:begin
                    X:=Round(Abs(Sz.CX*CA)+Abs(Sz.CY*SA));
                    Y:=Round(Abs(Sz.CY*CA))
                   end;
        2700:begin
              X:=Sz.CY;
              Y:=0
             end
       else
        begin
         X:=Round(Abs(Sz.CY*SA));
         Y:=0
        end
       end;
       SetTextColor(Canvas.Handle,ColorToRGB(FColors[Integer((Owner as TForm).Active)]));
       Canvas.TextOut(X,Y,Caption);
       SelectObject(Canvas.Handle,OldFont);
       DeleteObject(NewFont)
      end;

    procedure TCaptionText.Activate;
     begin
      HookOwner
     end;

    procedure TCaptionText.Deactivate;
     begin
      UnhookOwner
     end;

    {TFRManager}

    constructor TFRManager.Create;
     var I:Integer;
      begin
       if not (AOwner is TForm) then
        raise EOwnerMustBeForm.Create('The owner of TFRManager component must be the TForm descendant');
       with AOwner do
        for I:=0 to ComponentCount-1 do
         if (Components[I] is TFRManager) and (Components[I]<>Self) then
          raise EDuplicateComponent.Create('Only one TFRManager component on a form is allowed');
       inherited Create(AOwner);
       CallCount:=0
      end;

    procedure TFRManager.Activate;
     var I:Integer;
      begin
       if CallCount>0 then
        begin
         Dec(CallCount);
         if CallCount=0 then
          with Owner do
           for I:=0 to ComponentCount-1 do
            begin
             if Components[I] is TRegionComp then
              (Components[I] as TRegionComp).Activate;
             if Components[I] is THandlingControl then
              (Components[I] as THandlingControl).Activate;
             if Components[I] is TCaptionText then
              (Components[I] as TCaptionText).Activate
            end
        end
      end;

    procedure TFRManager.Deactivate;
     var I:Integer;
      begin
       Inc(CallCount);
       if CallCount=1 then
        with Owner do
         for I:=ComponentCount-1 downto 0 do
          begin
           if Components[I] is TRegionComp then
            (Components[I] as TRegionComp).Deactivate;
           if Components[I] is THandlingControl then
            (Components[I] as THandlingControl).Deactivate;
           if Components[I] is TCaptionText then
            (Components[I] as TCaptionText).Deactivate
          end
      end;

    procedure TFRManager.StayOnTop;
     begin
      Deactivate;
      if Stay then
       (Owner as TForm).FormStyle:=fsStayOnTop
      else
       (Owner as TForm).FormStyle:=fsNormal;
      Activate
     end;

    procedure Register;
     begin
      RegisterComponents('My Controls',[TFormRegion,TCaptionRegion,TTextExtRgn,TPictExtRgn,TMenuHotSpot,TSystemButton,TCaptionText,TFRManager]);
      RegisterPropertyEditor(TypeInfo(string),TRegionComp,'Region',TRegionProperty)
     end;

 initialization
  begin
   DFrame.X:=GetSystemMetrics(SM_CXDlgFrame);
   DFrame.Y:=GetSystemMetrics(SM_CYDlgFrame);
   SFrame.X:=GetSystemMetrics(SM_CXFixedFrame);
   SFrame.Y:=GetSystemMetrics(SM_CYFixedFrame);
   SzFrame.X:=GetSystemMetrics(SM_CXSizeFrame);
   SzFrame.Y:=GetSystemMetrics(SM_CYSizeFrame);
   NCapt:=GetSystemMetrics(SM_CYCaption);
   SmCapt:=GetSystemMetrics(SM_CYSmCaption)
  end;

 end.
